import string
import sys
import os
import ConfigParser

import pygtk
pygtk.require('2.0') # please comment this line for cx_Freeze
import gtk
# imports below for cx_Freeze
import gtk.glade
import gtk.gdk
import atk
import gtk._gtk
import pango


def mswindows():
  """Return TRUE is we run under Microsoft Windows"""
  return sys.platform == "win32"


def get_encoding():
    if (mswindows()):
      return "iso8859-1"
    try:
      enc = os.environ['LANG']
    except KeyError:  
      try:
        enc = os.environ['LC_LANG']
      except KeyError:  
        try:
          enc = os.environ['LC_ALL']
        except KeyError:  
          return "iso-8859-1"
    parts = string.split(enc, '.')
    if (len(parts) > 1) :
      return parts[1]
    return "iso-8859-1"


class RunSQL:
  """Execute a SQL query and fill the GtkTreeView with the results."""

  def __init__(self, execute, txtSQL, tabResult, result):
    self.execute = execute
    self.txtSQL = txtSQL
    self.lblResult = result
    self.tabResult = tabResult
    self.error_font = {}


  def result(self, msg):
    """update status bar text"""
    self.lblResult.set_markup('<span foreground="#004400">' + msg + '</span>')


  def set_proportional(self, widget):
    """Change the font of a widget to proportional"""
    font = widget.get_buffer().create_tag("RunFont" + str(widget))
    if (mswindows()):
      font.set_property('font', 'Courier New 10')
    else:
      font.set_property('font', 'Monospace 10')
    widget.ensure_style()
    return font


  def on_SetErrorFont(self, widget, event):
    o = widget.get_buffer()
    c1 = o.get_start_iter()
    c2 = o.get_end_iter()
    o.apply_tag(self.error_font[str(widget)],c1,c2)


  def str_limit(self, s, max = 20):
    s = string.replace(s, '\\\\', '\\')
    if (len(s) > (max + 3)):
      return s[:max] + "..."
    return s


  def error(self, msg, title = '<span foreground="#880000">Errors</span>'):
    """update the error messages"""
    scroll = gtk.ScrolledWindow()
    txt = gtk.TextView()
    txt.get_buffer().set_text(msg)
    self.error_font[str(txt)] = self.set_proportional(txt)
    txt.connect("event", self.on_SetErrorFont)
    txt.set_editable(gtk.FALSE)
    scroll.add(txt)
    scroll.show_all()
    label = gtk.Label()
    label.set_markup(title)
    self.tabResult.append_page(scroll, label)


  def add_treeview(self, title):
    """Add a new tree view"""
    scroll = gtk.ScrolledWindow()
    tvw = gtk.TreeView()
    scroll.add(tvw)
    scroll.show()
    label = gtk.Label(title)
    self.tabResult.append_page(scroll, label)
    return (tvw, label)
    

  def fill_treeview(self, sql, result):
    """build the tree view columns and rows"""
    sql = sql.strip()
    treeview, label = self.add_treeview(self.str_limit(sql))
    type_str = ""
    cell = gtk.CellRendererText()
    ncol = 0
    for col in result.description:
      column = gtk.TreeViewColumn(string.replace(col[0], "_", "__"))
      column.pack_start(cell, gtk.TRUE)
      column.add_attribute(cell, 'text', ncol)
      column.set_sort_column_id(ncol)
      treeview.append_column(column)
      ncol = ncol + 1
      if (type_str == ""):
        type_str = "str"
      else:  
        type_str = type_str + ",str"
      pass
      
    # dynamically create the liststore
    code = "liststore = gtk.ListStore(" + type_str + ")"
    exec compile(code, "<string>", 'exec')

    # fill the tree view
    row = result.fetchone()
    rows = 0
    while ( row is not None ) :
      liststore.append(row)
      row = result.fetchone()
      rows = rows + 1
    result.close()  
    treeview.set_model(liststore)
    treeview.set_reorderable(gtk.TRUE)
    treeview.columns_autosize()
    treeview.show()
    label.set_markup(label.get_text() + ' <span foreground="#555555">: ' + str(rows) + "</span>")
    return rows


  def run(self):  
    """Run the query and update the results"""
    # clear the treeviews and the error text buffers
    for child in self.tabResult.get_children():
      self.tabResult.remove(child)
    self.result("Please wait, executing the query and fetching the results...")
    # update the display
    while (gtk.events_pending() == gtk.TRUE):
      gtk.main_iteration_do(gtk.FALSE)

    # execute the query
    sqlbuffer = self.txtSQL.get_buffer()
    c1 = sqlbuffer.get_start_iter()
    c2 = sqlbuffer.get_end_iter()
    orig_sql = sqlbuffer.get_slice(c1, c2, gtk.TRUE)
    sql = self.execute.filter(orig_sql)
    notices = []
    try:
      result = self.execute.execute(sql)

      rows = 0
      if (isinstance(result, list)):
        # multiple queries and multiple results...
        parts = self.execute.split(orig_sql)
        sqls = []
        have_errors = 0
        for sql in parts :
          sql = string.strip(sql)
          if (len(sql) > 0) :
            sqls.append(sql)
        for res in result:
          sql = sqls.pop(0)
          try:
            if (res['cursor'].description is not None):
              rows += self.fill_treeview(sql, res['cursor'])
              notices = res['notices']
          except KeyError:    
            try:
              self.error(unicode(res['text'], get_encoding()), "psql : " + self.str_limit(sql))
            except KeyError:  
              notices = res['notices']
              self.error(res['error'], '<span foreground="#880000">Errors :</span> ' + self.str_limit(sql))
              have_errors += 1
        self.tabResult.set_current_page(0)
        if (rows > 1):
          res_str = str(rows) + " results"
        else:
          res_str = str(rows) + " result"
        if (have_errors > 1) :
          res_str += ', <span foreground="#880000">' + str(have_errors) + ' queries failed</span>'
        elif (have_errors > 0) :  
          res_str += ', <span foreground="#880000">' + str(have_errors) + ' query failed</span>'
        self.result(res_str)  
      else:
        try:
          if (result is None):
            self.result("No result")
          elif (result['cursor'].description is None):
            # not a SELECT
            notices = result['notices']
            self.result("No result")
          else:
            # only one SELECT
            rows = self.fill_treeview(sql, result['cursor'])
            notices = result['notices']
            if (rows > 1):
              self.result(str(rows) + " results")
            else:
              self.result(str(rows) + " result")
        except KeyError:
          try:
            self.error(unicode(result['text'], get_encoding()), "psql : " + self.str_limit(sql))
            self.result("")
          except KeyError:  
            self.error(result['error'])
            self.result('<span foreground="#880000">query failed</span>')
            notices = result['notices']
      buffer = self.txtSQL.get_buffer()
      buffer.move_mark_by_name('selection_bound', buffer.get_start_iter());
      buffer.move_mark_by_name('insert', buffer.get_end_iter());

    except Exception, errstr:
      self.result('<span foreground="#880000">query failed</span>')
      self.error(str(errstr))

    if (len(notices) > 0):
      msg = ""
      while len(notices):
        msg += notices.pop()
      self.error(msg, '<span foreground="#000088">log</span>')

    # restore the focus 
    self.txtSQL.grab_focus()

